@@ -1,19 +1,13 @@
 import json
 from typing import Any, Dict, Optional, Union
 
+from .const import CWT_CLAIM_NAMES
+
 
 class Claims:
-    """"""
-
-    REGISTERED_NAMES = {
-        "iss": 1,  # text string
-        "sub": 2,  # text string
-        "aud": 3,  # text string
-        "exp": 4,  # integer or floating-point number
-        "nbf": 5,  # integer or floating-point number
-        "iat": 6,  # integer or floating-point number
-        "cti": 7,  # byte string
-    }
+    """
+    CBOR Web Token (CWT) Claims Generator.
+    """
 
     def __init__(self, options: Optional[Dict[str, Any]] = None):
         """"""
@@ -21,7 +15,12 @@ class Claims:
         return
 
     def from_json(self, claims: Union[str, bytes, Dict[str, Any]]) -> Dict[int, Any]:
-        """"""
+        """
+        Convert JSON-formatted claims into CBOR-formatted claims
+        which has numeric keys.
+        If a key string in JSON data cannot be mapped to a numeric key,
+        it will be skipped.
+        """
         json_claims: Dict[str, Any]
         if isinstance(claims, str) or isinstance(claims, bytes):
             json_claims = json.loads(claims)
@@ -31,17 +30,23 @@ class Claims:
         for k in json_claims:
             if not isinstance(k, int):
                 break
-            ValueError("It is already CBOR-like format.")
+            raise ValueError("It is already CBOR-like format.")
 
         # Convert JSON to CBOR (Convert the type of key from str to int).
         cbor_claims = {}
         for k, v in json_claims.items():
-            if k not in Claims.REGISTERED_NAMES:
+            if k not in CWT_CLAIM_NAMES:
                 # TODO Support additional arguments.
                 continue
-            cbor_claims[Claims.REGISTERED_NAMES[k]] = v
-        if 7 in cbor_claims and isinstance(cbor_claims[7], str):
-            cbor_claims[7] = cbor_claims[7].encode("utf-8")
+            cbor_claims[CWT_CLAIM_NAMES[k]] = v
+
+        # Convert test string should be bstr into bstr.
+        # -259: EUPHNonce
+        # -258: EATMAROEPrefix
+        #    7: cti
+        for i in [-259, -258, 7]:
+            if i in cbor_claims and isinstance(cbor_claims[i], str):
+                cbor_claims[i] = cbor_claims[i].encode("utf-8")
         return cbor_claims
 
 
